package App::SeismicUnixGui::big_streams::check;

=head1 DOCUMENTATION

=head2 SYNOPSIS 

 PROGRAM NAME: check
 AUTHOR: Juan Lorenzo
 DATE:   Nov. 23 2022
 DESCRIPTION: 
 
 confirm value changes in 
 immodpg GUI
 
 Version: 0.0.1
 

=head2 USE

=head3 NOTES
	    

	  
=head4 
 Examples

=head3 

=head4 CHANGES and their DATES

V0.1 Nov 23, 2022

=cut

use Moose;
our $VERSION = '0.0.2';



1;